define(function(require, exports, module) {
    main.consumes = [
        "Plugin", "upload.manager", "c9", "proc", "notification.bubble",
        "tree", "menus", "ui", "dialog.error"
    ];

    main.provides = ["smartface.image"];
    return main;

    function main(options, imports, register) {
        const Plugin = imports.Plugin;
        const iOSimageNameRegex = /\w+(@(\dx)){0,1}/g;
        const plugin = new Plugin("Smartface", main.consumes, {
            name: "smartface.image"
        });
        const path = require("path");
        const c9 = imports.c9;
        const tree = imports.tree;
        const ui = imports.ui;
        const menus = imports.menus;
        const proc = imports.proc;
        const showError = imports["dialog.error"].show;
        const bubble = imports["notification.bubble"];
        const uploadmanager = imports["upload.manager"];
        const UPLOAD_FOLDER = "/images/automated/";
        const imagesRegexp = /images\/.*\.png/g;
        const uploadFolderRegexp = /images\/automated\/.*\.png/g;

        const STATE = {
            UPLOADING: "uploading",
            DONE: "done"
        };
        const SPECIAL_IMAGES = ['AppIcon', 'LaunchImage'];
        const imageNameRegexp = /(\w+)(\.(ios|android)){0,1}(\.(\dx)){0,1}\.png/g;
        const DEFAULT_SCALE_FACTOR = 3;
        const WORKSPACE_DIR = c9.workspaceDir;
        const DEST_FOLDER = {
            "iOS": "images/iOS",
            "Android": "images/Android"
        };

        var loaded = false;

        plugin.on("load", load);
        plugin.on("unload", unload);

        uploadmanager.on("addJob", function(event) {
            var imageName = path.basename(event.job.fullPath);
            if (!event.job.fullPath.startsWith(UPLOAD_FOLDER))
                return; //invalid folder.
            imageNameRegexp.lastIndex = 0;
            const result = imageNameRegexp.exec(imageName);
            var res = isValidImage(imageName);
            if (result) {
                imageName = result[1];
            }
            if (SPECIAL_IMAGES.indexOf(imageName) === -1 && !res.isValid) {
                event.job.cancel();
                showError({
                    html: createErrorMessage("Image Upload Error", res.image, res.errors.join("\n\t"))
                }, 7000);
            }
            /*event.job.on("changeState", function(e) {
                if (STATE.DONE === e.state) {
                    //console.log("Uploading state DONE.", e);
                    // Check path is appropriate ?
                }
            });
            */
        });

        function getImageNameForImageResizer(imageName) {
            iOSimageNameRegex.lastIndex = 0;
            var iOSImageRes = iOSimageNameRegex.exec(imageName);
            var res = imageName,
                scale = DEFAULT_SCALE_FACTOR;
            if (iOSImageRes && iOSImageRes[2]) {
                res = iOSImageRes[1] ? iOSImageRes[0].replace(iOSImageRes[1], "") : iOSImageRes[0];
                scale = parseFloat(iOSImageRes[2]) || DEFAULT_SCALE_FACTOR;
                res = res + "." + scale + "x";
            }
            return res;
        }

        function runSmartfaceImageProcessor(imageName, deleteFlag, callback) {
            var _rawImageName = imageName;
            var timer;
            imageNameRegexp.lastIndex = 0;
            const result = imageNameRegexp.exec(imageName);
            if (result) {
                _rawImageName = result[1];
            }
            var args = [
                "--srcImage=" + path.join(WORKSPACE_DIR, UPLOAD_FOLDER, imageName),
                "--destiOSFolder=" + path.join(WORKSPACE_DIR, DEST_FOLDER.iOS),
                "--destAndroidFolder=" + path.join(WORKSPACE_DIR, DEST_FOLDER.Android),
                "--delete=" + (deleteFlag ? "true" : ""),
                "--isIconImage=" + (SPECIAL_IMAGES[0] === _rawImageName ? "true" : ""),
                "--isLaunchImage=" + (SPECIAL_IMAGES[1] === _rawImageName ? "true" : "")
            ];
            proc.spawn("sfImageProcessor", {
                args: args
            }, function(err, process) {
                if (err) {
                    timer && clearInterval(timer);
                    return callback(err);
                }
                var data = "";
                process.stderr.on("data", function(chunk) {
                    data += chunk;
                });
                process.stdout.on("data", function(chunk) {
                    data += chunk;
                });
                process.on("exit", function(code, signal) {
                    timer && clearInterval(timer);
                    callback(null, { exitCode: code, data: data });
                });
            });

            timer = setInterval(function() {
                bubble.popup("Image generation is still working...");
            }, 5200);
        }

        function generateRelatedImages(imageName) {
            runSmartfaceImageProcessor(getImageNameForImageResizer(imageName),
                false,
                function(err, res) {
                    if (err || (res.exitCode !== 0)) {
                        showError({
                            html: createErrorMessage(null, imageName, res.data)
                        }, 8000);
                        return console.error(res.data);
                    }
                    console.log(res.data);
                    bubble.popup(imageName + " generation succeeded.");
                });
        }

        function load() {
            if (loaded)
                return;
            loaded = true;

            var dividerDiv1 = new ui.divider({
                visible: false
            });
            var menuItemDeletion = new ui.item({
                match: "file",
                caption: "Delete auto-resized images",
                isAvailable: function() {
                    imagesRegexp.lastIndex = 0;
                    const visible = imagesRegexp.test(tree.selectedNode.path);
                    menuItemDeletion.setAttribute("style", visible ? "display: '';" : "display: none;");
                    dividerDiv1.setAttribute("style", visible ? "display: '';" : "display: none;");
                    return true;
                },
                onclick: function() {
                    bubble.popup("deleting auto-resized images...");
                    runSmartfaceImageProcessor(
                        getImageNameForImageResizer(path.basename(tree.selectedNode.path)),
                        true,
                        function(err, res) {
                            if (err || (res.exitCode !== 0)) return console.error(err);
                            bubble.popup("auto-resized images deletion succeeded.");
                        });
                }
            });
            var menuItemGeneration = new ui.item({
                match: "file",
                caption: "Generate auto-resized images",
                isAvailable: function() {
                    uploadFolderRegexp.lastIndex = 0;
                    const visible = uploadFolderRegexp.test(tree.selectedNode.path);
                    menuItemGeneration.setAttribute("style", visible ? "display: '';" : "display: none;");
                    return true;
                },
                onclick: function(e) {
                    bubble.popup("generating auto-resized images...");
                    tree.selectedNodes.forEach(function(item) {
                        generateRelatedImages(path.basename(item.path));
                    });
                }
            });
            tree.getElement("mnuCtxTree", function(mnuCtxTree) {
                menus.addItemToMenu(mnuCtxTree, dividerDiv1, 141, plugin);
                menus.addItemToMenu(mnuCtxTree, menuItemGeneration, 142, plugin);
                menus.addItemToMenu(mnuCtxTree, menuItemDeletion, 143, plugin);
            });
        }

        function createErrorMessage(errorTitle, imageName, detail) {
            return "<div style=\"white-space: pre;\" >ERROR: " + (errorTitle || "Image Generation Error") +
                "</br>├─ image: " + imageName + " </br>└─ detail: " +
                detail + "</div>";
        }

        /**
         * Returns an object with following properties:
         * - isValid {bool}
         * - errors {string[]}
         * @param {string} relPath - Relative path of the image, something like:
         * images/iOS/smartface.png
         */
        function isValidImage(imageName) {
            var extension = path.extname(imageName);
            var imageNameRaw = path.basename(imageName, extension);
            var result = {
                image: imageName,
                isValid: true,
                errors: []
            };

            if (extension !== '.png') {
                result.isValid = false;
                result.errors.push("Invalid image format, only png supported.");
            }
            if (/\d/.test(imageName.charAt(0)) && imageName.charAt(0) !== '_') {
                result.isValid = false;
                result.errors.push("image name must begin with a small letter or an underscore.");
            }
            else if (!/^[a-z_][a-z0-9_]*([a-z0-9_]+)*$/.test(imageNameRaw)) {
                result.isValid = false;
                result.errors.push("image name can contain only lowercase letters, numbers and underscore.");
            }
            return result;
        }

        function unload() {
            loaded = false;
        }

        plugin.freezePublicAPI({});

        register(null, {
            "smartface.image": plugin
        });
    }
});
